@@ -1,19 +1,33 @@
 // SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 // SPDX-License-Identifier: BSD-3-Clause
 
+#include <vtkXMLUniformGridAMRWriter.h>
+
 #include "vtkCellData.h"
 #include "vtkCellIterator.h"
+#include "vtkCompositeDataIterator.h"
 #include "vtkConduitSource.h"
 #include "vtkImageData.h"
 #include "vtkLogger.h"
+#include "vtkMultiProcessController.h"
 #include "vtkNew.h"
+#include "vtkOverlappingAMR.h"
 #include "vtkPartitionedDataSet.h"
+#include "vtkPointData.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkSmartPointer.h"
 #include "vtkStructuredGrid.h"
+#include "vtkTestUtilities.h"
 #include "vtkUnstructuredGrid.h"
 #include "vtkVector.h"
 
+#if VTK_MODULE_ENABLE_VTK_ParallelMPI
+#include "vtkMPIController.h"
+#else
+#include "vtkDummyController.h"
+#endif
+
+#include <catalyst_conduit.hpp>
 #include <catalyst_conduit_blueprint.hpp>
 
 #define VERIFY(x, ...)                                                                             \
@@ -37,6 +51,9 @@ vtkSmartPointer<vtkDataObject> Convert(const conduit_cpp::Node& node)
 void CreateUniformMesh(
   unsigned int nptsX, unsigned int nptsY, unsigned int nptsZ, conduit_cpp::Node& res)
 {
+  auto controller = vtkMultiProcessController::GetGlobalController();
+  auto rank = controller->GetLocalProcessId();
+
   // Create the structure
   conduit_cpp::Node coords = res["coordsets/coords"];
   coords["type"] = "uniform";
@@ -52,7 +69,7 @@ void CreateUniformMesh(
   // -10 to 10 in each dim
   conduit_cpp::Node origin = coords["origin"];
   origin["x"] = -10.0;
-  origin["y"] = -10.0;
+  origin["y"] = -10.0 + 20 * rank;
 
   if (nptsZ > 1)
   {
@@ -76,6 +93,7 @@ bool ValidateMeshTypeUniform()
 {
   conduit_cpp::Node mesh;
   CreateUniformMesh(3, 3, 3, mesh);
+
   auto data = Convert(mesh);
   VERIFY(vtkPartitionedDataSet::SafeDownCast(data) != nullptr,
     "incorrect data type, expected vtkPartitionedDataSet, got %s", vtkLogIdentifier(data));
@@ -93,17 +111,11 @@ bool ValidateMeshTypeUniform()
   return true;
 }
 
-void CreateRectilinearMesh(
-  unsigned int nptsX, unsigned int nptsY, unsigned int nptsZ, conduit_cpp::Node& res)
+void GenerateValues(unsigned int nptsX, unsigned int nptsY, unsigned int nptsZ,
+  std::vector<double>& x, std::vector<double>& y, std::vector<double>& z)
 {
-  conduit_cpp::Node coords = res["coordsets/coords"];
-  coords["type"] = "rectilinear";
-
-  std::vector<double> x;
   x.resize(nptsX);
-  std::vector<double> y;
   y.resize(nptsY);
-  std::vector<double> z;
 
   if (nptsZ > 1)
   {
@@ -136,6 +148,18 @@ void CreateRectilinearMesh(
       z[k] = -10.0 + k * dz;
     }
   }
+}
+
+void CreateRectilinearMesh(
+  unsigned int nptsX, unsigned int nptsY, unsigned int nptsZ, conduit_cpp::Node& res)
+{
+  conduit_cpp::Node coords = res["coordsets/coords"];
+  coords["type"] = "rectilinear";
+
+  std::vector<double> x;
+  std::vector<double> y;
+  std::vector<double> z;
+  GenerateValues(nptsX, nptsY, nptsZ, x, y, z);
 
   conduit_cpp::Node coordVals = coords["values"];
   coordVals["x"].set(x);
@@ -168,6 +192,24 @@ bool ValidateMeshTypeRectilinear()
   VERIFY(dims[1] == 3, "incorrect y dimension expected=3, got=%d", dims[1]);
   VERIFY(dims[2] == 3, "incorrect z dimension expected=3, got=%d", dims[2]);
 
+  // Expected values
+  std::vector<double> x;
+  std::vector<double> y;
+  std::vector<double> z;
+  GenerateValues(3, 3, 3, x, y, z);
+  for (unsigned int i = 0; i < 3; i++)
+  {
+    VERIFY(x[i] == rg->GetXCoordinates()->GetComponent(i, 0),
+      "incorrect x value at %d: expected=%g, got=%g", i, x[i],
+      rg->GetXCoordinates()->GetComponent(i, 0));
+    VERIFY(y[i] == rg->GetYCoordinates()->GetComponent(i, 0),
+      "incorrect y value at %d: expected=%g, got=%g", i, y[i],
+      rg->GetYCoordinates()->GetComponent(i, 0));
+    VERIFY(z[i] == rg->GetZCoordinates()->GetComponent(i, 0),
+      "incorrect z value at %d: expected=%g, got=%g", i, z[i],
+      rg->GetZCoordinates()->GetComponent(i, 0));
+  }
+
   return true;
 }
 
@@ -356,14 +398,13 @@ bool ValidateMeshTypeUnstructured()
   return true;
 }
 
-bool CheckFieldDataMeshConversion(conduit_cpp::Node& mesh_node, int expected_number_of_arrays,
+bool CheckFieldData(vtkDataObject* data, int expected_number_of_arrays,
   const std::string& expected_array_name, int expected_number_of_components,
   std::vector<vtkVariant> expected_values)
 {
-  auto data = Convert(mesh_node);
   auto field_data = data->GetFieldData();
   VERIFY(field_data->GetNumberOfArrays() == expected_number_of_arrays,
-    "incorrect number of arrays in field data, expected 0, got %d",
+    "incorrect number of arrays in field data, expected %d, got %d", expected_number_of_arrays,
     field_data->GetNumberOfArrays());
 
   if (expected_number_of_arrays > 0)
@@ -371,7 +412,7 @@ bool CheckFieldDataMeshConversion(conduit_cpp::Node& mesh_node, int expected_num
     auto field_array = field_data->GetAbstractArray(0);
 
     VERIFY(std::string(field_array->GetName()) == expected_array_name,
-      "wrong array name, expected \"integer_field_data\", got %s", field_array->GetName());
+      "wrong array name, expected %s, got %s", expected_array_name.c_str(), field_array->GetName());
     VERIFY(field_array->GetNumberOfComponents() == expected_number_of_components,
       "wrong number of component");
     VERIFY(static_cast<size_t>(field_array->GetNumberOfTuples()) == expected_values.size(),
@@ -385,8 +426,95 @@ bool CheckFieldDataMeshConversion(conduit_cpp::Node& mesh_node, int expected_num
   return true;
 }
 
+bool CheckFieldDataMeshConversion(conduit_cpp::Node& mesh_node, int expected_number_of_arrays,
+  const std::string& expected_array_name, int expected_number_of_components,
+  std::vector<vtkVariant> expected_values)
+{
+  auto data = Convert(mesh_node);
+
+  CheckFieldData(data, expected_number_of_arrays, expected_array_name,
+    expected_number_of_components, expected_values);
+  auto pds = vtkPartitionedDataSet::SafeDownCast(data);
+  VERIFY(pds->GetNumberOfPartitions() == 1, "incorrect number of partitions, expected 1, got %d",
+    pds->GetNumberOfPartitions());
+  auto img = vtkImageData::SafeDownCast(pds->GetPartition(0));
+
+  CheckFieldData(img, expected_number_of_arrays, expected_array_name, expected_number_of_components,
+    expected_values);
+
+  return true;
+}
+
+bool ValidateMeshTypeAMR(const std::string& file)
+{
+  conduit_cpp::Node mesh;
+  // read in an example mesh dataset
+  conduit_node_load(conduit_cpp::c_node(&mesh), file.c_str(), "");
+
+  // add in point data
+  std::string field_name = "pointfield";
+  double field_value = 1;
+  size_t num_children = mesh["data"].number_of_children();
+  for (size_t i = 0; i < num_children; i++)
+  {
+    conduit_cpp::Node amr_block = mesh["data"].child(i);
+    int i_dimension = amr_block["coordsets/coords/dims/i"].to_int32();
+    int j_dimension = amr_block["coordsets/coords/dims/j"].to_int32();
+    int k_dimension = amr_block["coordsets/coords/dims/k"].to_int32();
+    conduit_cpp::Node fields = amr_block["fields"];
+    conduit_cpp::Node point_field = fields[field_name];
+    point_field["association"] = "vertex";
+    point_field["topology"] = "topo";
+    std::vector<double> point_values(
+      (i_dimension + 1) * (j_dimension + 1) * (k_dimension + 1), field_value);
+    point_field["values"] = point_values;
+  }
+
+  const auto& meshdata = mesh["data"];
+  // run vtk conduit source
+  vtkNew<vtkConduitSource> source;
+  source->SetUseAMRMeshProtocol(true);
+  source->SetNode(conduit_cpp::c_node(&meshdata));
+  source->Update();
+  auto data = source->GetOutputDataObject(0);
+
+  VERIFY(vtkOverlappingAMR::SafeDownCast(data) != nullptr,
+    "Incorrect data type, expected vtkOverlappingAMR, got %s", vtkLogIdentifier(data));
+
+  auto amr = vtkOverlappingAMR::SafeDownCast(data);
+
+  std::vector<double> bounds(6);
+  std::vector<double> origin(3);
+
+  amr->GetBounds(bounds.data());
+  amr->GetOrigin(0, 0, origin.data());
+
+  VERIFY(bounds[0] == 0 && bounds[1] == 1 && bounds[2] == 0 && bounds[3] == 1 && bounds[4] == 0 &&
+      bounds[5] == 1,
+    "Incorrect AMR bounds");
+
+  VERIFY(origin[0] == 0 && origin[1] == 0 && origin[2] == 0, "Incorrect AMR origin");
+
+  vtkSmartPointer<vtkCompositeDataIterator> iter;
+  iter.TakeReference(amr->NewIterator());
+  iter->InitTraversal();
+  for (iter->GoToFirstItem(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+  {
+    vtkDataSet* block = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
+    VERIFY(block->GetCellData()->GetArray("density") != nullptr, "Incorrect AMR cell data");
+    double range[2] = { -1, -1 };
+    block->GetPointData()->GetArray(field_name.c_str())->GetRange(range);
+    VERIFY(range[0] == field_value && range[1] == field_value, "Incorrect AMR point data");
+  }
+
+  return true;
+}
+
 bool ValidateFieldData()
 {
+  auto controller = vtkMultiProcessController::GetGlobalController();
+  auto rank = controller->GetLocalProcessId();
+
   conduit_cpp::Node mesh;
   CreateUniformMesh(3, 3, 3, mesh);
 
@@ -398,8 +526,8 @@ bool ValidateFieldData()
 
   field_data_node.remove(0);
   auto integer_field_data = field_data_node["integer_field_data"];
-  integer_field_data.set_int64(42);
-  VERIFY(CheckFieldDataMeshConversion(mesh, 1, integer_field_data.name(), 1, { 42 }),
+  integer_field_data.set_int64(42 + rank);
+  VERIFY(CheckFieldDataMeshConversion(mesh, 1, integer_field_data.name(), 1, { 42 + rank }),
     "Verification failed for integer field data.");
 
   field_data_node.remove(0);
@@ -439,7 +567,64 @@ bool ValidateFieldData()
   return true;
 }
 
-bool ValidateRectlinearGridWithDifferentDimensions()
+bool ValidateAscentGhostCellData()
+{
+  conduit_cpp::Node mesh;
+  CreateUniformMesh(3, 3, 3, mesh);
+
+  std::vector<int> cellGhosts(8, 0);
+  cellGhosts[2] = 1;
+
+  conduit_cpp::Node resCellFields = mesh["fields/ascent_ghosts"];
+  resCellFields["association"] = "element";
+  resCellFields["topology"] = "mesh";
+  resCellFields["volume_dependent"] = "false";
+  resCellFields["values"] = cellGhosts;
+
+  auto data = Convert(mesh);
+  auto pds = vtkPartitionedDataSet::SafeDownCast(data);
+  VERIFY(pds->GetNumberOfPartitions() == 1, "incorrect number of partitions, expected 1, got %d",
+    pds->GetNumberOfPartitions());
+  auto img = vtkImageData::SafeDownCast(pds->GetPartition(0));
+  VERIFY(img != nullptr, "missing partition 0");
+  vtkUnsignedCharArray* array = vtkUnsignedCharArray::SafeDownCast(
+    img->GetCellData()->GetArray(vtkDataSetAttributes::GhostArrayName()));
+  VERIFY(array != nullptr &&
+      array->GetValue(2) == static_cast<unsigned char>(vtkDataSetAttributes::HIDDENCELL),
+    "Verification failed for converting Ascent ghost cell data");
+
+  return true;
+}
+
+bool ValidateAscentGhostPointData()
+{
+  conduit_cpp::Node mesh;
+  CreateUniformMesh(3, 3, 3, mesh);
+
+  std::vector<int> pointGhosts(27, 0);
+  pointGhosts[2] = 1;
+
+  conduit_cpp::Node resPointFields = mesh["fields/ascent_ghosts"];
+  resPointFields["association"] = "vertex";
+  resPointFields["topology"] = "mesh";
+  resPointFields["values"] = pointGhosts;
+
+  auto data = Convert(mesh);
+  auto pds = vtkPartitionedDataSet::SafeDownCast(data);
+  VERIFY(pds->GetNumberOfPartitions() == 1, "incorrect number of partitions, expected 1, got %d",
+    pds->GetNumberOfPartitions());
+  auto img = vtkImageData::SafeDownCast(pds->GetPartition(0));
+  VERIFY(img != nullptr, "missing partition 0");
+  vtkUnsignedCharArray* array = vtkUnsignedCharArray::SafeDownCast(
+    img->GetPointData()->GetArray(vtkDataSetAttributes::GhostArrayName()));
+  VERIFY(array != nullptr &&
+      array->GetValue(2) == static_cast<unsigned char>(vtkDataSetAttributes::HIDDENPOINT),
+    "Verification failed for converting Ascent ghost point data");
+
+  return true;
+}
+
+bool ValidateRectilinearGridWithDifferentDimensions()
 {
   conduit_cpp::Node mesh;
   CreateRectilinearMesh(3, 2, 1, mesh);
@@ -901,12 +1086,29 @@ bool ValidateMeshTypeMixed()
 
 } // end namespace
 
-int TestConduitSource(int, char*[])
+int TestConduitSource(int argc, char** argv)
 {
-  return ValidateMeshTypeUniform() && ValidateMeshTypeRectilinear() &&
+#if VTK_MODULE_ENABLE_VTK_ParallelMPI
+  vtkNew<vtkMPIController> controller;
+#else
+  vtkNew<vtkDummyController> controller;
+#endif
+  controller->Initialize(&argc, &argv);
+  vtkMultiProcessController::SetGlobalController(controller);
+
+  std::string amrFile =
+    vtkTestUtilities::ExpandDataFileName(argc, argv, "Data/Conduit/bp_amr_example.json");
+
+  auto ret = ValidateMeshTypeUniform() && ValidateMeshTypeRectilinear() &&
       ValidateMeshTypeStructured() && ValidateMeshTypeUnstructured() && ValidateFieldData() &&
-      ValidateRectlinearGridWithDifferentDimensions() && Validate1DRectilinearGrid() &&
-      ValidateMeshTypeMixed() && ValidateMeshTypeMixed2D()
+      ValidateRectilinearGridWithDifferentDimensions() && Validate1DRectilinearGrid() &&
+      ValidateMeshTypeMixed() && ValidateMeshTypeMixed2D() && ValidateMeshTypeAMR(amrFile) &&
+      ValidateAscentGhostCellData() && ValidateAscentGhostPointData()
+
     ? EXIT_SUCCESS
     : EXIT_FAILURE;
+
+  controller->Finalize();
+
+  return ret;
 }
