@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkXOpenGLRenderWindow.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 // Must be included first to avoid conflicts with X11's `Status` define.
 #include "vtksys/SystemTools.hxx"
@@ -19,35 +7,6 @@
 #include "vtkOpenGLRenderer.h"
 #include "vtkXOpenGLRenderWindow.h"
 
-#include "vtk_glew.h"
-// Define GLX_GLXEXT_LEGACY to prevent glx.h from including the glxext.h
-// provided by the system.
-//#define GLX_GLXEXT_LEGACY
-
-// Ensure older version of glx.h define glXGetProcAddressARB
-#define GLX_GLXEXT_PROTOTYPES
-
-// New Workaround:
-// The GLX_GLXEXT_LEGACY definition was added to work around system glxext.h
-// files that used the GLintptr and GLsizeiptr types, but did not define them.
-// However, this broke multisampling (See PR#15433). Instead of using that
-// define, we're just defining the missing typedefs here.
-typedef ptrdiff_t GLintptr;
-typedef ptrdiff_t GLsizeiptr;
-#include "GL/glx.h"
-
-#ifndef GLAPI
-#define GLAPI extern
-#endif
-
-#ifndef GLAPIENTRY
-#define GLAPIENTRY
-#endif
-
-#ifndef APIENTRY
-#define APIENTRY GLAPIENTRY
-#endif
-
 #include "vtkCommand.h"
 #include "vtkIdList.h"
 #include "vtkImageData.h"
@@ -71,11 +30,15 @@ typedef ptrdiff_t GLsizeiptr;
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
+#include "vtk_glad.h"
+#include "vtkglad/include/glad/glx.h"
+
 /*
  * Work-around to get forward declarations of C typedef of anonymous
  * structs working. We do not want to include XUtil.h in the header as
  * it populates the global namespace.
  */
+VTK_ABI_NAMESPACE_BEGIN
 struct vtkXVisualInfo : public XVisualInfo
 {
 };
@@ -234,9 +197,10 @@ GLXFBConfig vtkXOpenGLRenderWindowGetDesiredFBConfig(Display* DisplayId, vtkType
 }
 
 template <int EventType>
-int XEventTypeEquals(Display*, XEvent* event, XPointer)
+int XEventTypeEquals(Display*, XEvent* event, XPointer winptr)
 {
-  return event->type == EventType;
+  return (event->type == EventType &&
+    *(reinterpret_cast<Window*>(winptr)) == reinterpret_cast<XAnyEvent*>(event)->window);
 }
 
 vtkXVisualInfo* vtkXOpenGLRenderWindow::GetDesiredVisualInfo()
@@ -244,18 +208,9 @@ vtkXVisualInfo* vtkXOpenGLRenderWindow::GetDesiredVisualInfo()
   XVisualInfo* v = nullptr;
 
   // get the default display connection
-  if (!this->DisplayId)
+  if (!this->EnsureDisplay())
   {
-    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
-
-    if (this->DisplayId == nullptr)
-    {
-      vtkErrorMacro(<< "bad X server connection. DISPLAY=" << vtksys::SystemTools::GetEnv("DISPLAY")
-                    << ". Aborting.\n");
-      abort();
-    }
-
-    this->OwnDisplay = 1;
+    return nullptr;
   }
   this->Internal->FBConfig =
     vtkXOpenGLRenderWindowGetDesiredFBConfig(this->DisplayId, this->StereoCapableWindow,
@@ -302,6 +257,16 @@ vtkXOpenGLRenderWindow::vtkXOpenGLRenderWindow()
   this->XCSizeSW = 0;
   this->XCHand = 0;
   this->XCCustom = 0;
+
+  auto loadFunc = [](void*, const char* name) -> VTKOpenGLAPIProc
+  {
+    if (name)
+    {
+      return glXGetProcAddress((const GLubyte*)name);
+    }
+    return nullptr;
+  };
+  this->SetOpenGLSymbolLoader(loadFunc, nullptr);
 }
 
 // free up memory & close the window
@@ -367,9 +332,10 @@ void vtkXOpenGLRenderWindow::SetStereoCapableWindow(vtkTypeBool capable)
 }
 
 static int PbufferAllocFail = 0;
+#define vtkXOGLPbufferErrorHandler VTK_ABI_NAMESPACE_MANGLE(vtkXOGLPbufferErrorHandler)
 extern "C"
 {
-  int vtkXOGLPbufferErrorHandler(Display*, XErrorEvent*)
+  int VTK_ABI_NAMESPACE_MANGLE(vtkXOGLPbufferErrorHandler)(Display*, XErrorEvent*)
   {
     PbufferAllocFail = 1;
     return 1;
@@ -377,9 +343,11 @@ extern "C"
 }
 
 static bool ctxErrorOccurred = false;
+#define vtkXOGLContextCreationErrorHandler                                                         \
+  VTK_ABI_NAMESPACE_MANGLE(vtkXOGLContextCreationErrorHandler)
 extern "C"
 {
-  int vtkXOGLContextCreationErrorHandler(Display*, XErrorEvent*)
+  int VTK_ABI_NAMESPACE_MANGLE(vtkXOGLContextCreationErrorHandler)(Display*, XErrorEvent*)
   {
     ctxErrorOccurred = true;
     return 1;
@@ -407,7 +375,8 @@ void vtkXOpenGLRenderWindow::SetShowWindow(bool val)
       if (winattr.map_state == IsUnmapped)
       {
         XEvent e;
-        XIfEvent(this->DisplayId, &e, XEventTypeEquals<MapNotify>, nullptr);
+        XIfEvent(this->DisplayId, &e, XEventTypeEquals<MapNotify>,
+          reinterpret_cast<XPointer>(&this->WindowId));
       }
       this->Mapped = 1;
     }
@@ -422,7 +391,8 @@ void vtkXOpenGLRenderWindow::SetShowWindow(bool val)
       if (winattr.map_state != IsUnmapped)
       {
         XEvent e;
-        XIfEvent(this->DisplayId, &e, XEventTypeEquals<UnmapNotify>, nullptr);
+        XIfEvent(this->DisplayId, &e, XEventTypeEquals<UnmapNotify>,
+          reinterpret_cast<XPointer>(&this->WindowId));
       }
       this->Mapped = 0;
     }
@@ -456,16 +426,9 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
   xsh.height = height;
 
   // get the default display connection
-  if (!this->DisplayId)
+  if (!this->EnsureDisplay())
   {
-    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
-    if (this->DisplayId == nullptr)
-    {
-      vtkErrorMacro(<< "bad X server connection. DISPLAY=" << vtksys::SystemTools::GetEnv("DISPLAY")
-                    << ". Aborting.\n");
-      abort();
-    }
-    this->OwnDisplay = 1;
+    return;
   }
 
   attr.override_redirect = False;
@@ -481,8 +444,8 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     v = this->GetDesiredVisualInfo();
     if (!v)
     {
-      vtkErrorMacro(<< "Could not find a decent visual\n");
-      abort();
+      vtkWarningMacro(<< "Could not find a decent visual\n");
+      return;
     }
     this->ColorMap = XCreateColormap(
       this->DisplayId, XRootWindow(this->DisplayId, v->screen), v->visual, AllocNone);
@@ -554,7 +517,7 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
   // is GLX extension is supported?
   if (!glXQueryExtension(this->DisplayId, nullptr, nullptr))
   {
-    vtkErrorMacro("GLX not found.  Aborting.");
+    vtkWarningMacro("GLX not found.");
     if (this->HasObserver(vtkCommand::ExitEvent))
     {
       this->InvokeEvent(vtkCommand::ExitEvent, nullptr);
@@ -562,7 +525,7 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     }
     else
     {
-      abort();
+      return;
     }
   }
 
@@ -576,7 +539,12 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
         (const GLubyte*)"glXCreateContextAttribsARB");
 
     int context_attribs[] = { GLX_CONTEXT_MAJOR_VERSION_ARB, 3, GLX_CONTEXT_MINOR_VERSION_ARB, 2,
-      // GLX_CONTEXT_FLAGS_ARB        , GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB,
+    // GLX_CONTEXT_FLAGS_ARB        , GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB,
+#ifdef GL_ES_VERSION_3_0
+      GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_ES_PROFILE_BIT_EXT,
+#else
+      GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
+#endif
       0 };
 
     if (glXCreateContextAttribsARB)
@@ -598,7 +566,12 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
 
       // we believe that these later versions are all compatible with
       // OpenGL 3.2 so get a more recent context if we can.
+      // For GLES, version 3.0 is best supported by VTK shaders.
+#ifdef GL_ES_VERSION_3_0
+      int attemptedVersions[] = { 3, 0 };
+#else
       int attemptedVersions[] = { 4, 5, 4, 4, 4, 3, 4, 2, 4, 1, 4, 0, 3, 3, 3, 2 };
+#endif
 
       // try shared context first, the fallback to not shared
       bool done = false;
@@ -648,7 +621,7 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
 
   if (!this->Internal->ContextId)
   {
-    vtkErrorMacro("Cannot create GLX context.  Aborting.");
+    vtkWarningMacro("Cannot create GLX context.");
     if (this->HasObserver(vtkCommand::ExitEvent))
     {
       this->InvokeEvent(vtkCommand::ExitEvent, nullptr);
@@ -656,7 +629,7 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     }
     else
     {
-      abort();
+      return;
     }
   }
 
@@ -666,7 +639,8 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     XMapWindow(this->DisplayId, this->WindowId);
     XSync(this->DisplayId, False);
     XEvent e;
-    XIfEvent(this->DisplayId, &e, XEventTypeEquals<MapNotify>, nullptr);
+    XIfEvent(this->DisplayId, &e, XEventTypeEquals<MapNotify>,
+      reinterpret_cast<XPointer>(&this->WindowId));
     XGetWindowAttributes(this->DisplayId, this->WindowId, &winattr);
     // if the specified window size is bigger than the screen size,
     // we have to reset the window size to the screen size
@@ -808,6 +782,10 @@ void vtkXOpenGLRenderWindow::DestroyWindow()
 void vtkXOpenGLRenderWindow::WindowInitialize()
 {
   this->CreateAWindow();
+  if (!this->DisplayId || !this->WindowId)
+  {
+    return;
+  }
 
   this->MakeCurrent();
 
@@ -834,6 +812,10 @@ void vtkXOpenGLRenderWindow::Initialize()
 
 void vtkXOpenGLRenderWindow::Finalize()
 {
+  if (!this->Initialized)
+  {
+    return;
+  }
   // clean and destroy window
   this->DestroyWindow();
 }
@@ -963,20 +945,62 @@ void vtkXOpenGLRenderWindow::SetSize(int width, int height)
         this->Interactor->SetSize(width, height);
       }
 
+      // get baseline serial number for X requests generated from XResizeWindow
+      unsigned long serial = NextRequest(this->DisplayId);
+
+      // request a new window size from the X server
       XResizeWindow(this->DisplayId, this->WindowId, static_cast<unsigned int>(width),
         static_cast<unsigned int>(height));
-      // this is an async call so we wait until we know it has been resized.
+
+      // flush output queue and wait for X server to processes the request
       XSync(this->DisplayId, False);
-      XWindowAttributes attribs;
-      XGetWindowAttributes(this->DisplayId, this->WindowId, &attribs);
-      if (attribs.width != width || attribs.height != height)
+
+      // The documentation for XResizeWindow includes this important note:
+      //
+      //   If the override-redirect flag of the window is False and some
+      //   other client has selected SubstructureRedirectMask on the parent,
+      //   the X server generates a ConfigureRequest event, and no further
+      //   processing is performed.
+      //
+      // What this means, essentially, is that if this window is a top-level
+      // window, then it's the window manager (the "other client") that is
+      // responsible for changing this window's size.  So when we call
+      // XResizeWindow() on a top-level window, then instead of resizing
+      // the window immediately, the X server informs the window manager,
+      // and then the window manager sets our new size (usually it will be
+      // the size we asked for).  We receive a ConfigureNotify event when
+      // our new size has been set.
+
+      // check our override-redirect flag
+      XWindowAttributes attrs;
+      XGetWindowAttributes(this->DisplayId, this->WindowId, &attrs);
+      if (!attrs.override_redirect && this->ParentId)
       {
-        XEvent e;
-        XIfEvent(this->DisplayId, &e, XEventTypeEquals<ConfigureNotify>, nullptr);
+        // check if parent has SubstructureRedirectMask
+        XWindowAttributes parentAttrs;
+        XGetWindowAttributes(this->DisplayId, this->ParentId, &parentAttrs);
+        if ((parentAttrs.all_event_masks & SubstructureRedirectMask) == SubstructureRedirectMask)
+        {
+          // set the wait timeout to be 2 seconds from now
+          double maxtime = 2.0 + vtksys::SystemTools::GetTime();
+          // look for a ConfigureNotify that came *after* XResizeWindow
+          XEvent e;
+          while (!XCheckIfEvent(this->DisplayId, &e, XEventTypeEquals<ConfigureNotify>,
+                   reinterpret_cast<XPointer>(&this->WindowId)) ||
+            e.xconfigure.serial < serial)
+          {
+            // wait for 10 milliseconds and try again until time runs out
+            vtksys::SystemTools::Delay(10);
+            if (vtksys::SystemTools::GetTime() > maxtime)
+            {
+              vtkWarningMacro(<< "Timeout while waiting for response to XResizeWindow.");
+              return;
+            }
+          }
+          XPutBackEvent(this->DisplayId, &e);
+        }
       }
     }
-
-    this->Modified();
   }
 }
 
@@ -1161,9 +1185,11 @@ void vtkXOpenGLRenderWindow::SetForceMakeCurrent()
 
 vtkTypeBool vtkXOpenGLRenderWindowFoundMatch;
 
+#define vtkXOpenGLRenderWindowPredProc VTK_ABI_NAMESPACE_MANGLE(vtkXOpenGLRenderWindowPredProc)
 extern "C"
 {
-  Bool vtkXOpenGLRenderWindowPredProc(Display* vtkNotUsed(disp), XEvent* event, char* arg)
+  Bool VTK_ABI_NAMESPACE_MANGLE(vtkXOpenGLRenderWindowPredProc)(
+    Display* vtkNotUsed(disp), XEvent* event, char* arg)
   {
     Window win = (Window)arg;
 
@@ -1209,19 +1235,11 @@ vtkTypeBool vtkXOpenGLRenderWindow::GetEventPending()
 int* vtkXOpenGLRenderWindow::GetScreenSize()
 {
   // get the default display connection
-  if (!this->DisplayId)
+  if (!this->EnsureDisplay())
   {
-    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
-    if (this->DisplayId == nullptr)
-    {
-      vtkErrorMacro(<< "bad X server connection. DISPLAY=" << vtksys::SystemTools::GetEnv("DISPLAY")
-                    << ". Aborting.\n");
-      abort();
-    }
-    else
-    {
-      this->OwnDisplay = 1;
-    }
+    this->ScreenSize[0] = 0;
+    this->ScreenSize[1] = 0;
+    return this->ScreenSize;
   }
 
   this->ScreenSize[0] = XDisplayWidth(this->DisplayId, XDefaultScreen(this->DisplayId));
@@ -1262,6 +1280,29 @@ Display* vtkXOpenGLRenderWindow::GetDisplayId()
   return this->DisplayId;
 }
 
+bool vtkXOpenGLRenderWindow::EnsureDisplay()
+{
+  if (!this->DisplayId)
+  {
+    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
+    if (this->DisplayId == nullptr)
+    {
+      // Only warn about DISPLAY if on-screen rendering is selected,
+      // this helps with automatic detection of best window backend.
+      if (this->ShowWindow)
+      {
+        vtkWarningMacro(<< "bad X server connection. DISPLAY="
+                        << vtksys::SystemTools::GetEnv("DISPLAY"));
+      }
+    }
+    else
+    {
+      this->OwnDisplay = 1;
+    }
+  }
+  return this->DisplayId != nullptr;
+}
+
 // Get this RenderWindow's parent X window id.
 Window vtkXOpenGLRenderWindow::GetParentId()
 {
@@ -1326,26 +1367,11 @@ void vtkXOpenGLRenderWindow::SetWindowId(Window arg)
 // Set this RenderWindow's X window id to a pre-existing window.
 void vtkXOpenGLRenderWindow::SetWindowInfo(const char* info)
 {
+  // note: potential Display/Window mismatch here
+  this->EnsureDisplay();
+
   int tmp;
-
-  // get the default display connection
-  if (!this->DisplayId)
-  {
-    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
-    if (this->DisplayId == nullptr)
-    {
-      vtkErrorMacro(<< "bad X server connection. DISPLAY=" << vtksys::SystemTools::GetEnv("DISPLAY")
-                    << ". Aborting.\n");
-      abort();
-    }
-    else
-    {
-      this->OwnDisplay = 1;
-    }
-  }
-
   sscanf(info, "%i", &tmp);
-
   this->SetWindowId(static_cast<Window>(tmp));
 }
 
@@ -1354,33 +1380,17 @@ void vtkXOpenGLRenderWindow::SetNextWindowInfo(const char* info)
 {
   int tmp;
   sscanf(info, "%i", &tmp);
-
   this->SetNextWindowId(static_cast<Window>(tmp));
 }
 
 // Sets the X window id of the window that WILL BE created.
 void vtkXOpenGLRenderWindow::SetParentInfo(const char* info)
 {
+  // note: potential Display/Window mismatch here
+  this->EnsureDisplay();
+
   int tmp;
-
-  // get the default display connection
-  if (!this->DisplayId)
-  {
-    this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
-    if (this->DisplayId == nullptr)
-    {
-      vtkErrorMacro(<< "bad X server connection. DISPLAY=" << vtksys::SystemTools::GetEnv("DISPLAY")
-                    << ". Aborting.\n");
-      abort();
-    }
-    else
-    {
-      this->OwnDisplay = 1;
-    }
-  }
-
   sscanf(info, "%i", &tmp);
-
   this->SetParentId(static_cast<Window>(tmp));
 }
 
@@ -1760,3 +1770,4 @@ void vtkXOpenGLRenderWindow::SetCurrentCursor(int shape)
       break;
   }
 }
+VTK_ABI_NAMESPACE_END
